@@ -28,11 +28,9 @@ import shutil
 import unittest
 import tempfile
 
-PY3K = sys.version_info[0] == 3
-
-import gi
-gi.require_version('GExiv2', '0.10')
-from gi.repository import GExiv2
+import gi.overrides
+gi.overrides.__path__.insert(0, os.path.join(os.path.dirname(__file__), '..', '..'))
+from gi.repository import GExiv2, GLib
 from fractions import Fraction
 
 
@@ -185,13 +183,25 @@ class TestMetadata(unittest.TestCase):
                        'stop_emission',
                        'stop_emission_by_name',
                        'thaw_notify',
+                       'try_clear_tag',
+                       'try_tag_supports_multiple_values',
                        'try_delete_gps_info',
+                       'try_erase_exif_thumbnail',
                        'try_generate_xmp_packet',
+                       'try_get_comment',
                        'try_get_exif_tag_rational',
+                       'try_get_exposure_time',
+                       'try_get_fnumber',
+                       'try_get_focal_length',
                        'try_get_gps_altitude',
                        'try_get_gps_info',
                        'try_get_gps_latitude',
                        'try_get_gps_longitude',
+                       'try_get_iso_speed',
+                       'try_get_metadata_pixel_height',
+                       'try_get_metadata_pixel_width',
+                       'try_get_orientation',
+                       'try_get_preview_image',
                        'try_get_tag_description',
                        'try_get_tag_interpreted_string',
                        'try_get_tag_label',
@@ -200,13 +210,23 @@ class TestMetadata(unittest.TestCase):
                        'try_get_tag_raw',
                        'try_get_tag_string',
                        'try_get_tag_type',
+                       'try_get_xmp_namespace_for_tag',
                        'try_get_xmp_packet',
+                       'try_has_tag',
+                       'try_register_xmp_namespace',
+                       'try_set_comment',
                        'try_set_exif_tag_rational',
+                       'try_set_exif_thumbnail_from_buffer',
                        'try_set_gps_info',
+                       'try_set_metadata_pixel_height',
+                       'try_set_metadata_pixel_width',
+                       'try_set_orientation',
                        'try_set_tag_long',
                        'try_set_tag_multiple',
                        'try_set_tag_string',
                        'try_set_xmp_tag_struct',
+                       'try_unregister_all_xmp_namespaces',
+                       'try_unregister_xmp_namespace',
                        'try_update_gps_info',
                        'unref',
                        'unregister_all_xmp_namespaces',
@@ -216,8 +236,90 @@ class TestMetadata(unittest.TestCase):
                        'weak_ref']:
             self.assertIn(method, methods)
 
+        preview_properties = GExiv2.PreviewProperties()
+        methods = [m for m in dir(preview_properties) if not m.startswith('_')]
+        for method in ['bind_property',
+                       'bind_property_full',
+                       'chain',
+                       'connect',
+                       'connect_after',
+                       'connect_object',
+                       'connect_object_after',
+                       'disconnect',
+                       'disconnect_by_func',
+                       'emit',
+                       'emit_stop_by_name',
+                       'force_floating',
+                       'freeze_notify',
+                       'g_type_instance',
+                       'get_extension',
+                       'get_height',
+                       'get_mime_type',
+                       'get_qdata',
+                       'get_size',
+                       'get_width',
+                       'interface_find_property',
+                       'interface_install_property',
+                       'interface_list_properties',
+                       'notify',
+                       'ref_count',
+                       'ref_sink',
+                       'replace_data',
+                       'replace_qdata',
+                       'set_property',
+                       'steal_data',
+                       'steal_qdata',
+                       'thaw_notify',
+                       'unref',
+                       'watch_closure',
+                       'weak_ref'
+                       ]:
+            self.assertIn(method, methods)
+
+        preview_image = GExiv2.PreviewImage()
+        methods = [m for m in dir(preview_image) if not m.startswith('_')]
+        for method in ['bind_property',
+                       'bind_property_full',
+                       'chain',
+                       'connect',
+                       'connect_after',
+                       'connect_object',
+                       'connect_object_after',
+                       'disconnect',
+                       'disconnect_by_func',
+                       'emit',
+                       'emit_stop_by_name',
+                       'force_floating',
+                       'freeze_notify',
+                       'g_type_instance',
+                       'get_data',
+                       'get_extension',
+                       'get_mime_type',
+                       'get_height',
+                       'get_width',
+                       'get_qdata',
+                       'interface_find_property',
+                       'interface_install_property',
+                       'interface_list_properties',
+                       'notify',
+                       'ref_count',
+                       'ref_sink',
+                       'replace_data',
+                       'replace_qdata',
+                       'set_property',
+                       'steal_data',
+                       'steal_qdata',
+                       'thaw_notify',
+                       'try_write_file',
+                       'unref',
+                       'watch_closure',
+                       'write_file',
+                       'weak_ref'
+                       ]:
+            self.assertIn(method, methods)
+
     def test_open_buf(self):
-        with open(self.get_input_file(), 'rb' if PY3K else 'r') as fd:
+        with open(self.get_input_file(), 'rb') as fd:
             buf = fd.read()
         metadata = GExiv2.Metadata()
         metadata.open_buf(buf)
@@ -660,7 +762,7 @@ generated the image. When the field is left blank, it is treated as unknown.""")
         self.assertEqual(len(thumb), 36660)
 
     def test_set_exif_thumbnail_from_buffer(self):
-        with open(self.get_input_file(), 'rb' if PY3K else 'r') as fd:
+        with open(self.get_input_file(), 'rb') as fd:
             buf = fd.read()
         self.metadata.set_exif_thumbnail_from_buffer(buf)
         thumb = self.metadata.get_exif_thumbnail()
@@ -669,7 +771,7 @@ generated the image. When the field is left blank, it is treated as unknown.""")
     def test_erase_exif_thumbnail(self):
         self.metadata.erase_exif_thumbnail()
         thumb = self.metadata.get_exif_thumbnail()
-        self.assertEqual(thumb, b'' if PY3K else '')
+        self.assertEqual(thumb, b'')
 
     def test_has_xmp(self):
         self.assertFalse(self.metadata.has_xmp())
@@ -722,6 +824,41 @@ generated the image. When the field is left blank, it is treated as unknown.""")
         self.assertEqual(
             self.metadata.get_iptc_tags(), ['Iptc.Application2.City'])
 
+    def test_try_tag_supports_multiple_values(self):
+        # Test different types of built-in/added tags for Exif/Iptc/Xmp
+         
+        # Add a new xmpText string
+        self.metadata.set_tag_string('Xmp.dc.NewXmpTextValue', 'New Value')
+        
+        ### Test Exif tags
+        self.assertFalse(self.metadata.try_tag_supports_multiple_values('Exif.Image.Model'))
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'Exif.Madeup.Orientation')
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'Exif.Image.MadeUp')
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'Exif.MadeUp.MadeUp')
+
+        ### Test Iptc tags
+        self.assertTrue(self.metadata.try_tag_supports_multiple_values('Iptc.Application2.Keywords'))
+        self.assertFalse(self.metadata.try_tag_supports_multiple_values('Iptc.Envelope.ServiceId'))
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'Iptc.Madeup.Keywords')
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'Iptc.Application2.Madeup')
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'Iptc.Madeup.MadeUp')
+
+        ### Test Xmp tags
+        self.assertFalse(self.metadata.try_tag_supports_multiple_values('Xmp.xmpMM.InstanceID'))
+        self.assertTrue(self.metadata.try_tag_supports_multiple_values('Xmp.dc.subject'))
+        self.assertTrue(self.metadata.try_tag_supports_multiple_values('Xmp.photoshop.TextLayers'))
+        self.assertTrue(self.metadata.try_tag_supports_multiple_values('Xmp.xmpRights.UsageTerms'))
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'Xmp.MadeUp.subject')
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'Xmp.dc.MadeUp')
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'Xmp.MadeUp.MadeUp')
+        self.assertFalse(self.metadata.try_tag_supports_multiple_values('Xmp.dc.NewXmpTextValue'))
+
+        ### Test Other tags
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values, '')
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'MadeUp.dc.subject')
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'MadeUp.MadeUp.subject')
+        self.assertRaises(GLib.GError,self.metadata.try_tag_supports_multiple_values,'MadeUp.MadeUp.MadeUp')
+
     def test_delete_gps_info(self):
         # Longitude, latitude, altitude
         self.metadata.set_gps_info(-123.35, 48.43, 10)
@@ -756,6 +893,14 @@ generated the image. When the field is left blank, it is treated as unknown.""")
         self.assertAlmostEqual(lat, 48.43, 9)
         self.assertEqual(alt, 10)
 
+    def test_try_get_gps_info_without_altitude(self):
+        self.metadata.set_gps_info(-1.0, 2.0, 3.0)
+        self.metadata.clear_tag('Exif.GPSInfo.GPSAltitudeRef')
+        lon, lat, alt = self.metadata.try_get_gps_info()
+        self.assertAlmostEqual(lon, -1.0, 9)
+        self.assertAlmostEqual(lat, 2.0, 9)
+        self.assertEqual(alt, 0.0)
+
     def test_set_gps_info(self):
         # Longitude, latitude, altitude
         self.metadata.set_gps_info(-123.35, 48.43, 10)
@@ -875,3 +1020,22 @@ generated the image. When the field is left blank, it is treated as unknown.""")
         self.assertEqual(self.metadata.get_comment(), 'Lego Dragon, Copenhagen.')
         self.metadata.clear_comment()
         self.assertEqual(self.metadata.get_comment().strip(), '')
+
+    def test_try_get_xmp_namespace_for_tag(self):
+        self.assertEqual(self.metadata.try_get_xmp_namespace_for_tag('dc'), 'http://purl.org/dc/elements/1.1/')
+        self.assertEqual(self.metadata.try_get_xmp_namespace_for_tag('Xmp.dc.subject'), 'http://purl.org/dc/elements/1.1/')
+        self.assertEqual(self.metadata.try_get_xmp_namespace_for_tag('Xmp.dc.MadeUp'), 'http://purl.org/dc/elements/1.1/')
+
+        # Invalid "familyName"
+        self.assertEqual(self.metadata.get_xmp_namespace_for_tag('xmp.dc.subject'), None)
+        self.assertEqual(self.metadata.get_xmp_namespace_for_tag('MadeUp.dc.subject'), None)
+        self.assertEqual(self.metadata.get_xmp_namespace_for_tag('.dc.subject'), None)
+
+        # Invalid "groupName"
+        self.assertEqual(self.metadata.get_xmp_namespace_for_tag('MadeUp'), None)
+        self.assertEqual(self.metadata.get_xmp_namespace_for_tag('Xmp.MadeUp.subject'), None)
+        self.assertEqual(self.metadata.get_xmp_namespace_for_tag('Xmp..subject'), None)
+
+        # Missing "tagName"
+        self.assertEqual(self.metadata.get_xmp_namespace_for_tag('Xmp.dc'), None)
+        self.assertEqual(self.metadata.get_xmp_namespace_for_tag('Xmp.dc.'), None)
